@@ -106,8 +106,8 @@ LearnerRegrDebug = R6Class("LearnerRegrDebug", inherit = LearnerRegr,
 
       if (self$predict_type == "quantiles") {
         prediction = list(quantiles = matrix(self$model$quantiles, nrow = n, ncol = length(self$model$quantiles), byrow = TRUE))
-        attr(prediction$quantiles, "probs") = self$model$quantile_probs
-        attr(prediction$quantiles, "response") = self$quantile_response
+        setattr(prediction$quantiles, "probs", self$model$quantile_probs)
+        setattr(prediction$quantiles, "response", self$quantile_response)
         return(prediction)
       }
 
