@@ -18,7 +18,7 @@ fix_factor_levels.data.table = function(data, levels, ...) { # nolint
 fix_factor_levels.Matrix = function(data, levels, ...) { # nolint
   levels = levels[intersect(names(levels), names(data))]
   if (length(levels)) {
-    stop("Factor handling in Matrix data is not supported")
+    stopf("Factor handling in Matrix data is not supported")
   }
   data
 }
