@@ -15,9 +15,10 @@
 #' See [mlr3misc::Dictionary].
 #'
 #' @section S3 methods:
-#' * `as.data.table(dict)`\cr
+#' * `as.data.table(dict, ..., objects = FALSE)`\cr
 #'   [mlr3misc::Dictionary] -> [data.table::data.table()]\cr
-#'   Returns a [data.table::data.table()] with columns `"key"`, `"params"`, and `"iters"`.
+#'   Returns a [data.table::data.table()] with columns "key", "label", "params", and "iters".
+#'   If `objects` is set to `TRUE`, the constructed objects are returned in the list column named `object`.
 #'
 #' @family Dictionary
 #' @family Resampling
@@ -34,7 +35,8 @@ mlr_resamplings = R6Class("DictionaryResampling",
 )$new()
 
 #' @export
-as.data.table.DictionaryResampling = function(x, ...) { # nolint
+as.data.table.DictionaryResampling = function(x, ..., objects = FALSE) { # nolint
+  assert_flag(objects)
 
   setkeyv(map_dtr(x$keys(), function(key) {
     r = tryCatch(x$get(key),
@@ -43,6 +45,9 @@ as.data.table.DictionaryResampling = function(x, ...) { # nolint
       return(list(key = key))
     }
 
-    list(key = key, params = list(r$param_set$ids()), iters = r$iters)
+    insert_named(
+      list(key = key, label = r$label, params = list(r$param_set$ids()), iters = r$iters),
+      if (objects) list(object = list(r))
+    )
   }, .fill = TRUE), "key")[]
 }
