@@ -1,12 +1,13 @@
 #' @import data.table
 #' @import checkmate
+#' @import cli
 #' @import paradox
 #' @import mlr3misc
 #' @import palmerpenguins
 #' @importFrom R6 R6Class is.R6
 #' @importFrom utils data head tail getFromNamespace packageVersion
 #' @importFrom graphics plot
-#' @importFrom stats predict rnorm runif sd contr.treatment model.frame terms quantile
+#' @importFrom stats predict rnorm runif sd contr.treatment model.frame terms quantile weighted.mean
 #' @importFrom uuid UUIDgenerate
 #' @importFrom parallelly availableCores
 #' @importFrom future nbrOfWorkers plan
@@ -52,6 +53,8 @@
 #'   Note that results computed in debug mode use a different seeding mechanism and are **not reproducible**.
 #' * `"mlr3.warn_version_mismatch"`: Set to `FALSE` to silence warnings raised during predict if a learner has been
 #'   trained with a different version version of mlr3.
+#' * `"mlr3.prob_as_default"`: Set to `TRUE` to set the predict type of classification learners to
+#'   `"prob"` by default (if they support it).
 #'
 #' @references
 #' `r tools::toRd(citation("mlr3"))`
@@ -76,7 +79,7 @@ dummy_import = function() {
   x$add("mlr3.holdout_task", load_callback_holdout_task)
 
   # setup logger
-  lg = lgr::get_logger(pkgname)
+  lg = lgr::get_logger("mlr3/core")
   assign("lg", lg, envir = parent.env(environment()))
   f = function(event) {
     event$msg = paste0("[mlr3] ", event$msg)
@@ -90,8 +93,6 @@ dummy_import = function() {
   register_namespace_callback(pkgname, "mlr", function(...) {
     warning("Packages 'mlr3' and 'mlr' are conflicting and should not be loaded in the same session")
   })
-
-  mlr_reflections$loggers[["mlr3"]] = lg
 } # nocov end
 
 leanify_package()
