@@ -5,7 +5,7 @@ test_that("autotest", {
   expect_true(result, info = result$error)
 
   exclude = c("formula", "data", "weights", "subset", "na.action", "method", "model",
-    "x", "y", "parms", "control", "cost", "keep_model")
+    "x", "y", "parms", "control", "cost", "keep_model", "use_weights")
   result = run_paramtest(learner, list(rpart::rpart, rpart::rpart.control), exclude, tag = "train")
   expect_true(result, info = result$error)
 
@@ -36,21 +36,16 @@ test_that("selected_features", {
   expect_subset(sf, task$feature_names, empty.ok = FALSE)
 })
 
-
-test_that("weights", {
+test_that("use_weights actually influences the model", {
   task = TaskClassif$new("foo", as_data_backend(cbind(iris, data.frame(w = rep(c(1, 10, 100), each = 50)))), target = "Species")
-  task$set_col_roles("w", character())
-  learner = lrn("classif.rpart")
-
+  task$set_col_roles("w", "weights_learner")
+  learner = lrn("classif.rpart", use_weights = "use")
   learner$train(task)
   c1 = learner$predict(task)$confusion
-
-  task$set_col_roles("w", "weight")
+  learner = lrn("classif.rpart", use_weights = "ignore")
   learner$train(task)
   c2 = learner$predict(task)$confusion
-
-  expect_true(sum(c1[1:2, 3]) > 0)
-  expect_true(sum(c2[1:2, 3]) == 0)
+  expect_false(all(c1 == c2))
 })
 
 test_that("default_values on rpart", {
